# *-* coding: utf-8 *-*

import os
import socket
import config_utils
from errors import ServiceMethodError
from flask import current_app
from tuner import Tuner
from lftp_mirror import get_ftp_status


def is_broadcasting(tuner_id):
    resp = _socket_command(tuner_id, 'is broadcasting')
    if resp == '1':
        return True
    else:
        return False


def is_writing(tuner_id):
    resp = _socket_command(tuner_id, 'is writing')
    if resp == '1':
        return True
    else:
        return False


def start_broadcast(tuner_id):
    resp = _socket_command(tuner_id, 'start broadcast')
    if resp == 'ok':
        return True
    else:
        return False


def stop_broadcast(tuner_id):
    resp = _socket_command(tuner_id, 'stop broadcast')
    if resp == 'ok':
        return True
    else:
        return False


def get_runtime_status(tuner_id):
    logger = current_app.logger
    client = _conn_socket(tuner_id)

    try:
        if _send_socket_command(client, 'is broadcasting') == '1':
            broadcast = Tuner.BroadcastStatus.BROADCASTING
        else:
            broadcast = Tuner.BroadcastStatus.DOES_NOT_BROADCASTING
    except Exception, e:
        broadcast = Tuner.BroadcastStatus.UNKNOWN
        logger.error("Cant't get broadcast status", exc_info=e)

    try:
        if _send_socket_command(client, 'is writing') == '1':
            writing = Tuner.WritingStatus.WRITING
        else:
            writing = Tuner.WritingStatus.DOES_NOT_WRITING
    except Exception, e:
        writing = Tuner.WritingStatus.UNKNOWN
        logger.error("Cant't get writing status", exc_info=e)

    try:
        quality_abs = int(_send_socket_command(client, 'get quality'))
        quality = _normalize_signal_quality_level(quality_abs)
    except Exception, e:
        quality_abs = 0
        quality = -1
        logger.error("Cant't get quality", exc_info=e)

    try:
        volume_abs = float(_send_socket_command(client, 'get volume'))
        volume = _normalize_signal_volume_level(volume_abs)
    except Exception, e:
        volume = -1
        volume_abs = -90
        logger.error("Cant't get volume", exc_info=e)

    _close_socket(client)
    current_app.logger.info('Tuner %s is: broadcasting: %s, writing: %s, quality: %s, volume: %s', tuner_id, broadcast,
                            writing, quality, volume_abs)
    ftp_status = get_ftp_status(tuner_id)
    return writing, broadcast, quality, quality_abs, volume, volume_abs, ftp_status


def _socket_command(tuner_id, command):
    client = _conn_socket(tuner_id)
    resp = _send_socket_command(client, command)
    _close_socket(client)
    return resp


def _conn_socket(tuner_id):
    logger = current_app.logger
    socket_path = config_utils.get_tuner_socket_file(tuner_id)
    if not os.path.exists(socket_path):
        raise ServiceMethodError("Socket %s doesn't exist" % socket_path)
    client = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    logger.debug("Connecting to unix socket %s", socket_path)
    client.settimeout(1)
    client.connect(socket_path)
    client.settimeout(0.5)
    return client


def _close_socket(client):
    current_app.logger.debug("Closing socket")
    client.close()


def _send_socket_command(client, command):
    logger = current_app.logger
    logger.debug("Sending to socket '%s'", command)

#    ready = select.select([client], [], [], 0.5)
    client.send(command)
    resp = client.recv(1024)
#    if ready[0]:
#        resp = client.recv(1024)
#    else:
#        raise Exception("socket timed out")
    logger.debug("Received from socket '%s'", resp)

    if resp == 'error' or resp == 'no such command':
        raise Exception
    return resp


def _normalize_signal_quality_level(quality):
    levels = config_utils.get_signal_quality_levels()
    for ind in range(0, len(levels)):
        if quality < levels[ind]:
            return ind
    return len(levels)


def _normalize_signal_volume_level(volume):
    levels = config_utils.get_signal_volume_levels()
    for ind in range(0, len(levels)):
        if volume < levels[ind]:
            return ind
    return len(levels)
