'use strict';

angular.module('fmtapper.services', ['ngResource'])
.factory('NotificationService', function($parse) {
    return {
        notify: function(msg, type) {
            $.notify(msg, {
                placement: {
                    from: 'top',
                    align: 'center'
                },
                type: type
            });
        }
    };
})
.factory('FormService', function($parse) {
    var GLOBAL_ERROR_FIELD_NAME = 'globalError';
  return {
		  emitServerErrors: function(scope, formName, data, basePrefix) {
			  	scope.serverErrors = {};

			  	var emitError = function(fieldName, msg) {
                        var clearedFieldName;
                        if(fieldName == GLOBAL_ERROR_FIELD_NAME
                                || !scope[formName][fieldName]
                                || !scope[formName][fieldName].$setValidity) {
                            clearedFieldName = fieldName.replace(/\[\d+]/g, '');
                            scope[formName].$setValidity(clearedFieldName, false, scope[formName]);
                        }
                        else {
                            clearedFieldName = fieldName;
                            scope[formName][clearedFieldName].$setValidity('server', false);
                        }
                        if(scope[formName][clearedFieldName]) {
                            scope[formName][clearedFieldName].$error.serverMessage = msg;
                        }
                        else {
                            scope[formName][clearedFieldName] = {$error: {serverMessage: msg}};
                        }
                    },
                    walkErrorFields = function(prefix, obj) {
                        var getFieldName = function(prefix, name) {
                            return prefix ? prefix + '.' + name : name;
                            };
                        _.each(obj, function(value, key) {
                            if(angular.isArray(value)){
                                var msg = value.join('\n');
                                emitError(getFieldName(prefix, key), msg);
                            }
                            else if(angular.isObject(value)) {
                               walkErrorFields(getFieldName(prefix, key), value);
                            }
                            else {
                               emitError(getFieldName(prefix, key), value);
                            }
                        });
                    }

			  	var globalErrors = data['global_errors'];
			  	if(globalErrors) {
			  		emitError(GLOBAL_ERROR_FIELD_NAME, globalErrors.join('\n'));
			  	}

              var fieldErrors = data['field_errors'];
              if(fieldErrors) {
                walkErrorFields(basePrefix, fieldErrors);
              }
		  },
		  clearServerErrors: function(scope, formName, prefix) {
              _.each(scope[formName], function(value, key) {
                  if(key.indexOf(prefix) == 0) {
                     if(scope[formName][key].$setValidity) {
                         scope[formName][key].$setValidity('server', true);
                     }
                     delete scope[formName][key].$error.serverMessage;
                  }
              });

              if(scope[formName][GLOBAL_ERROR_FIELD_NAME]) {
                  delete scope[formName][GLOBAL_ERROR_FIELD_NAME].$error.serverMessage;
                  if(scope[formName][GLOBAL_ERROR_FIELD_NAME].$setValidity) {
                      scope[formName][GLOBAL_ERROR_FIELD_NAME].$setValidity('server', true);
                  }
              }
		  }
	  };
})
.factory('FileService', function($sce) {
    var findDir = function(path, parent) {
            if(parent.path == path) {
                return parent;
            }
            else {
                var child = _.find(parent.children, function(c) {return c.children && path.indexOf(c.path) == 0})
                if(child) {
                    return findDir(path, child);
                }
                else {
                    return null;
                }
            }
        };

    return {
        listDir: function(path, parentDir) {
            var dir = findDir(path, parentDir);
            var fileItems;
            if(dir) {
                fileItems = dir.children;
            }
            else {
                fileItems = [];
            }
            return fileItems;
        },
        updateFileLinks: function(dir, linkFunction) {
            var self = this;
            _.each(dir.children, function(c) {
                if(!c.children) {
                    c.link = $sce.trustAsResourceUrl(linkFunction(c.path));
                }
                else {
                    self.updateFileLinks(c, linkFunction);
                }
            });
        },
        getParentDir: function(path) {
            var res;
            if(path) {
                var ind = path.indexOf('/');
                if(ind) {
                    res = path.substr(0, ind);
                }
                else {
                    res = "";
                }
            }
            else {
                res = ".";
            }
            return res;
        }
    };
})
.factory('IntervalService', function($sce) {
    var padLeft = function (nr, n, str){
        return Array(n-String(nr).length+1).join(str||'0')+nr;
    };
    return {
        format: function (intervalStr) {
            var res = /(\d\d):(\d\d):(\d\d)/.exec(intervalStr);
            if(!res)
                return
            var minutes = res[3];
            var hours = res[2];
            var days = res[1];
            if(days != 0) {
                if(days > 31)
                    days = 31;
                return padLeft(days, 2) + ':00:00';
            }
            if(hours != 0) {
                if(hours > 23)
                    hours = 23;
                return  '00:' + padLeft(hours, 2) + ':00';
            }
            if(minutes != 0) {
                if(minutes > 59)
                    minutes = 59;
                return '00:00:' + padLeft(minutes, 2);
            }
        }
    };
});
